/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

// A simple caosdb client
#include <iostream>
#include <memory>
#include <string>
#include "caosdb/info/v1alpha1/main.pb.h"
#include "constants.h"
#include "connection.h"
#include "authentication.h"
#include "utils.h"

auto main() -> int {

  std::cout << "CaosDB (libcaosdb " << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_PATCH << ")" << std::endl;
  std::cout << "We don't miss the H of caos." << std::endl;

  const auto pem_file = caosdb::utils::get_env_var("CAOSDB_SERVER_CA_PEM", std::string());
  const auto host = caosdb::utils::get_env_var("CAOSDB_SERVER_HOST", "localhost");
  const auto port_str = caosdb::utils::get_env_var("CAOSDB_SERVER_PORT", "8000");
  const auto port = std::stoi(port_str);

  auto auth = std::make_shared<caosdb::authentication::PlainPasswordAuthenticator>("tf", "vuY36moa");
  auto cacert = std::make_shared<caosdb::PemFileCACertProvider>(pem_file);
  auto config =
    std::make_shared<caosdb::SslCaosDBConnectionConfig>(host, port, cacert, auth);
  caosdb::CaosDBConnection connection(config);
  std::cout << std::endl << connection << std::endl;
  const auto &v_info = connection.getVersionInfo();
  // const auto &build = v_info.build();
  // std::cout << "BUILD(" << build[0] << ")" << std::endl;
  std::cout << "VersionInfo(" << v_info.major() << "." << v_info.minor() << "."
            << v_info.patch() << ")" << std::endl;
  return 0;
}
